/*----------------------------------------------------------------------------
 * Name:    LED.c
 * Purpose: low level LED functions
 * Note(s):
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2009-2014 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <LPC15xx.h>                         /* LPC15xx definitions           */
#include "LED.h"

#define LED_NUM     4                        /* Number of user LEDs          */

const unsigned long led_port[] = {       1,       1,       0,       1 };
const unsigned long led_mask[] = { 1UL<< 2, 1UL<< 3, 1UL<< 0, 1UL<< 0 };


/*-----------------------------------------------------------------------------
 *      LED_Initialize:  Initialize LEDs
 *
 * Parameters: (none)
 * Return:     (none)
 *----------------------------------------------------------------------------*/
 void LED_Initialize (void) {

  LPC_SYSCON->SYSAHBCLKCTRL0 |= ((1UL << 14) | (1UL << 15));  /* enable clock for GPIO0 and GPIO1 port */

  /* configure GPIO as output */
  LPC_GPIO_PORT->DIR[0]  |= led_mask[2];
  LPC_GPIO_PORT->SET[0]  |= led_mask[2];
  LPC_GPIO_PORT->DIR[1]  |= (led_mask[0] | led_mask[1] | led_mask[3]);
  LPC_GPIO_PORT->SET[1]  |= (led_mask[0] | led_mask[1] | led_mask[3]);

}

/*-----------------------------------------------------------------------------
 *      LED_Uninitialize:  Uninitialize LEDs
 *
 * Parameters: (none)
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Uninitialize (void) {
  LPC_SYSCON->SYSAHBCLKCTRL0 &= ~((1UL << 14) | (1UL << 15)); /* disable clock for GPIO1 port */

  /* configure GPIO as output */
  LPC_GPIO_PORT->DIR[0]  &= ~led_mask[2];
  LPC_GPIO_PORT->SET[0]  &= ~led_mask[2];
  LPC_GPIO_PORT->DIR[1]  &= ~(led_mask[0] | led_mask[1] | led_mask[3]);
  LPC_GPIO_PORT->SET[1]  &= ~(led_mask[0] | led_mask[1] | led_mask[3]);

}

/*-----------------------------------------------------------------------------
 *      LED_On: Turns on requested LED
 *
 * Parameters:  num - LED number
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_On (unsigned int num) {

  if (num < LED_NUM) {
    LPC_GPIO_PORT->CLR[led_port[num]] |= led_mask[num];
  }
}

/*-----------------------------------------------------------------------------
 *      LED_Off: Turns off requested LED
 *
 * Parameters:  num - LED number
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Off (unsigned int num) {

  if (num < LED_NUM) {
    LPC_GPIO_PORT->SET[led_port[num]]  = led_mask[num];
  }
}

/*-----------------------------------------------------------------------------
 *      LED_Out: Write value to LEDs
 *
 * Parameters:  val - value to be displayed on LEDs
 * Return:     (none)
 *----------------------------------------------------------------------------*/
void LED_Out(unsigned int value) {
  int i;

  for (i = 0; i < LED_NUM; i++) {
    if (value & (1<<i)) {
      LED_On (i);
    } else {
      LED_Off(i);
    }
  }
}

/*-----------------------------------------------------------------------------
 *      LED_Num: Get number of available LEDs
 *
 * Parameters: (none)
 * Return:      number of available LEDs
 *----------------------------------------------------------------------------*/
uint32_t LED_Num (void) {
  return (LED_NUM);
}
