/*********************************************************************
*                SEGGER Microcontroller GmbH & Co. KG                *
*        Solutions for real time microcontroller applications        *
**********************************************************************
*                                                                    *
*        (c) 1996 - 2014  SEGGER Microcontroller GmbH & Co. KG       *
*                                                                    *
*        Internet: www.segger.com    Support:  support@segger.com    *
*                                                                    *
**********************************************************************

** emWin V5.24 - Graphical user interface for embedded applications **
All  Intellectual Property rights  in the Software belongs to  SEGGER.
emWin is protected by  international copyright laws.  Knowledge of the
source code may not be used to write a similar product.  This file may
only be used in accordance with the following terms:

The software has been licensed to  ARM LIMITED whose registered office
is situated at  110 Fulbourn Road,  Cambridge CB1 9NJ,  England solely
for  the  purposes  of  creating  libraries  for  ARM7, ARM9, Cortex-M
series,  and   Cortex-R4   processor-based  devices,  sublicensed  and
distributed as part of the  MDK-ARM  Professional  under the terms and
conditions  of  the   End  User  License  supplied  with  the  MDK-ARM
Professional. 
Full source code is available at: www.segger.com

We appreciate your understanding and fairness.
----------------------------------------------------------------------
File        : LCD_X_SPI_LPC15xx.c
Purpose     : Port routines for LPC15xx SPI
----------------------------------------------------------------------
*/

#include "GUI.h"

/*********************************************************************
*
*       Hardware configuration
*
**********************************************************************
*/

#include <LPC15xx.h>                         /* LPC15xx definitions           */


/*********************** Hardware specific configuration **********************/

/* SPI Interface: SPI1

   PINS:
   - CS     = P0.25 (GPIO pin)
   - RS     = GND
   - WR/SCK = P1.10 (SCK1)
   - RD     = GND
   - SDO    = P1.11 (MISO1)
   - SDI    = P1.9  (MOSI1)                                                    */

#define PIN_CS      (1 << 25)
#define PIN_CLK     (1 << 10)
#define PIN_DAT     (1 <<  9)

#define IN          0x00
#define OUT         0x01

/* SPI_SR - bit definitions                                                   */
#define RNE         0x01



/*--------------- Graphic LCD interface hardware definitions -----------------*/

/* Pin CS setting to 0 or 1                                                   */
#define LCD_CS(x)   ((x) ? (LPC_GPIO_PORT->SET[0] = PIN_CS)    : (LPC_GPIO_PORT->CLR[0] = PIN_CS))
#define LCD_CLK(x)  ((x) ? (LPC_GPIO_PORT->SET[1] = PIN_CLK)   : (LPC_GPIO_PORT->CLR[1] = PIN_CLK))
#define LCD_DAT(x)  ((x) ? (LPC_GPIO_PORT->SET[1] = PIN_DAT)   : (LPC_GPIO_PORT->CLR[1] = PIN_DAT))

#define DAT_MODE(x) ((x == OUT) ? (LPC_GPIO_PORT->DIR[1] |= PIN_DAT) : (LPC_GPIO_PORT->DIR[1] &= ~PIN_DAT))
#define BUS_VAL()                ((LPC_GPIO_PORT->PIN[1]  & PIN_DAT) != 0)


#define SPI_START   (0x70)              /* Start byte for SPI transfer        */
#define SPI_RD      (0x01)              /* WR bit 1 within start              */
#define SPI_WR      (0x00)              /* WR bit 0 within start              */
#define SPI_DATA    (0x02)              /* RS bit 1 within start byte         */
#define SPI_INDEX   (0x00)              /* RS bit 0 within start byte         */


/************************ Local auxiliary functions ***************************/




/*******************************************************************************
* Transfer 1 byte over the serial communication                                *
*   Parameter:    byte:   byte to be sent                                      *
*   Return:               byte read while sending                              *
*******************************************************************************/

static __inline unsigned char spi_tran (unsigned char byte) {

  LPC_SPI1->TXDAT = byte;
  while (!(LPC_SPI1->STAT & RNE));        /* Wait for send to finish            */
  return (LPC_SPI1->RXDAT);
}



/*********************************************************************
*
*       Exported code
*
*********************************************************************
*/
/*********************************************************************
*
*       LCD_X_Init
*
* Purpose:
*   This routine should be called from your application program
*   to set port pins to their initial values
*/
void LCD_X_Init(void) {

  LPC_SYSCON->SYSAHBCLKCTRL0 |= ((1UL << 15) |   /* enable clock for GPIO1    */
                                 (1UL << 14) |   /* enable clock for GPIO0    */
                                 (1UL << 12)  ); /* enable clock for SWM      */

  LPC_SYSCON->SYSAHBCLKCTRL1 |=  (1UL << 10);    /* enable clock for SPI1     */

  /* Configure the LCD Control pins  (Backlight)                              */
  LPC_GPIO_PORT->DIR[0] |= (1UL << 10);
  LPC_GPIO_PORT->CLR[0]  = (1UL << 10);

  /* SSEL1 is GPIO output set to high                                         */
  LPC_GPIO_PORT->DIR[0] |= (1UL << 25);
  LPC_GPIO_PORT->SET[0]  = (1UL << 25);

  /* Configure the SPI Pins                                                   */
  LPC_SWM->PINASSIGN5 &= ~((0xFF <<  0) |        /* clear SPI1 pin assignment */
                           (0xFF <<  8) |
                           (0xFF << 16)  );
  LPC_SWM->PINASSIGN5 |=  ((  42 <<  0) |        /* P1.10 SCK                 */
                           (  41 <<  8) |        /* P1.9  MOSI                */
                           (  43 << 16)  );      /* P1.11 MISO                */

  /* Enable SPI in Master Mode, CPOL=1, CPHA=1                                */
  /* Max. 50 MBit used for Data Transfer @ 72MHz                              */
  LPC_SPI1->CFG    = ((1UL <<  0) |              /* SPI enable                */
                      (1UL <<  2) |              /* master mode               */
                      (1UL <<  4) |              /* CPHA  = 1                 */
                      (1UL <<  5)  );            /* CPOL  = 1                 */
  LPC_SPI1->TXCTL =   (7UL << 24);               /* 8bit dataframe            */
  LPC_SPI1->DIV   =    2;                        /* clk = PCLK/3 = 24MHz      */

  LPC_GPIO_PORT->SET[0]  = (1UL << 10);          /* Backlight On*/
}
/*********************************************************************
*
*       LCD_X_ClrCS
*
* Purpose:
*   Sets the chip select pin to low
*/
void LCD_X_ClrCS(void) {
  LCD_CS(0);
}

/*********************************************************************
*
*       LCD_X_SetCS
*
* Purpose:
*   Sets the chip select pin to high
*/
void LCD_X_SetCS(void) {
  LCD_CS(1);
}


/*********************************************************************
*
*       LCD_X_WriteM
*
* Purpose:
*   Writes multiple bytes to controller
*/
void LCD_X_WriteM(U8 * pData, int NumBytes) {
  while (NumBytes--) {
    spi_tran(*pData++);
  }
}
/*********************************************************************
*
*       LCD_X_ReadM
*
* Purpose:
*   Reads multiple bytes from the controller
*/
void LCD_X_ReadM(U8 * pData, int NumBytes) {
  while (NumBytes--) {
    *pData++ = spi_tran(0);
  }
}

/*************************** End of file ****************************/
